@@ -1,21 +1,21 @@
 #' Retrieve an attribute.
 #'
-#' \code{xml_attrs()} retrieves all attributes values as a named character
-#' vector, \code{xml_attrs() <-} sets all attribute values. \code{xml_attr()}
-#' retrieves the value of single attribute and \code{xml_attr() <-} modifies
-#' its value. If the attribute doesn't exist, it will return \code{default},
-#' which defaults to \code{NA}. \code{xml_has_attr()} tests if an attribute is
-#' present.
+#' `xml_attrs()` retrieves all attributes values as a named character
+#' vector, `xml_attrs() <-` or `xml_set_attrs()` sets all attribute
+#' values. `xml_attr()` retrieves the value of single attribute and
+#' `xml_attr() <-` or `xml_set_attr()` modifies its value. If the
+#' attribute doesn't exist, it will return `default`, which defaults to
+#' `NA`. `xml_has_attr()` tests if an attribute is present.
 #'
 #' @inheritParams xml_name
 #' @param attr Name of attribute to extract.
 #' @param default Default value to use when attribute is not present.
-#' @return \code{xml_attr()} returns a character vector. \code{NA} is used
+#' @return `xml_attr()` returns a character vector. `NA` is used
 #'  to represent of attributes that aren't defined.
 #'
-#'  \code{xml_has_attr()} returns a logical vector.
+#'  `xml_has_attr()` returns a logical vector.
 #'
-#'  \code{xml_attrs()} returns a named character vector if \code{x} x is single
+#'  `xml_attrs()` returns a named character vector if `x` x is single
 #'  node, or a list of character vectors if given a nodeset
 #' @export
 #' @examples
@@ -51,6 +51,14 @@
 #' xml_attr(doc, "id")
 #' xml_attr(doc, "b:id", ns)
 #' xml_attr(doc, "id", ns)
+#'
+#' # Can set a single attribute with `xml_attr() <-` or `xml_set_attr()`
+#' xml_attr(doc, "id") <- "one"
+#' xml_set_attr(doc, "id", "two")
+#'
+#' # Or set multiple attributes with `xml_attrs()` or `xml_set_attrs()`
+#' xml_attrs(doc) <- c("b:id" = "one", "f:id" = "two", "id" = "three")
+#' xml_set_attrs(doc, c("b:id" = "one", "f:id" = "two", "id" = "three"))
 xml_attr <- function(x, attr, ns = character(), default = NA_character_) {
   UseMethod("xml_attr")
 }
@@ -110,8 +118,9 @@ xml_attrs.xml_nodeset <- function(x, ns = character()) {
 #' @export
 `xml_attr<-.xml_node` <- function(x, attr, ns = character(), value) {
   if (is.null(value)) {
-    node_set_attr(x$node, name = attr, nsMap = ns, "")
+    node_remove_attr(x$node, name = attr, nsMap = ns)
   } else {
+    value <- as.character(value)
     node_set_attr(x$node, name = attr, nsMap = ns, value)
   }
   x
@@ -119,10 +128,44 @@ xml_attrs.xml_nodeset <- function(x, ns = character()) {
 
 #' @export
 `xml_attr<-.xml_nodeset` <- function(x, attr, ns = character(), value) {
-  lapply(x, `xml_attr<-`, attr = attr, ns = ns, value = value)
+  if (length(x) == 0) {
+    return(x)
+  }
+
+  if (length(value) == 0) {
+    value <- list(value)
+  }
+
+  mapply(`xml_attr<-`, x, attr = attr, value = value, SIMPLIFY = FALSE, MoreArgs = list(ns = ns))
   x
 }
 
+#' @export
+`xml_attr<-.xml_missing` <- function(x, attr, ns = character(), value) {
+  x
+}
+
+#' @rdname xml_attr
+#' @export
+xml_set_attr <- function(x, attr, value, ns = character()) {
+  UseMethod("xml_set_attr")
+}
+
+# This function definition is used for all methods, we need to rearrange the `ns`
+# argument to be at the end of the set function
+set_attr_fun <- function(x, attr, value, ns = character()) {
+  xml_attr(x = x, attr = attr, ns = ns) <- value
+}
+
+#' @export
+xml_set_attr.xml_node <- set_attr_fun
+
+#' @export
+xml_set_attr.xml_nodeset <- set_attr_fun
+
+#' @export
+xml_set_attr.xml_missing <- set_attr_fun
+
 #' @rdname xml_attr
 #' @export
 `xml_attrs<-` <- function(x, ns = character(), value) {
@@ -137,6 +180,9 @@ xml_attrs.xml_nodeset <- function(x, ns = character()) {
 
   attrs <- names(value)
 
+  # as.character removes all attributes (including names)
+  value <- stats::setNames(as.character(value), attrs)
+
   current_attrs <- names(xml_attrs(x, ns = ns))
 
   existing <- intersect(current_attrs, attrs)
@@ -159,6 +205,9 @@ xml_attrs.xml_nodeset <- function(x, ns = character()) {
 
 #' @export
 `xml_attrs<-.xml_nodeset` <- function(x, ns = character(), value) {
+  if (length(x) == 0) {
+    return(x)
+  }
   if (!is.list(ns)) {
      ns <- list(ns)
   }
@@ -174,3 +223,28 @@ xml_attrs.xml_nodeset <- function(x, ns = character()) {
   x
 }
 
+#' @export
+`xml_attrs<-.xml_missing` <- function(x, ns = character(), value) {
+  x
+}
+
+#' @rdname xml_attr
+#' @export
+xml_set_attrs <- function(x, value, ns = character()) {
+  UseMethod("xml_set_attrs")
+}
+
+# This function definition is used for all methods, we need to rearrange the `ns`
+# argument to be at the end of the set function
+set_attrs_fun <- function(x, value, ns = character()) {
+  xml_attrs(x = x, ns = ns) <- value
+}
+
+#' @export
+xml_set_attrs.xml_node <- set_attrs_fun
+
+#' @export
+xml_set_attrs.xml_nodeset <- set_attrs_fun
+
+#' @export
+xml_set_attrs.xml_missing <- set_attrs_fun
